// Copyright 2019, University of Freiburg,
// Chair of Algorithms and Data Structures.
// Authors: Patrick Brosi <brosi@informatik.uni-freiburg.de>

#ifndef OSMFIXER_INDEX_STATIDX_H_
#define OSMFIXER_INDEX_STATIDX_H_

#include <map>
#include <string>
#include <vector>
#include "util/geo/Geo.h"
#include "util/geo/Grid.h"

namespace osmfixer {

typedef std::map<std::string, std::vector<std::string>> OsmAttrs;

struct AttrErr {
  std::string attr;
  std::string otherAttr;
  double conf;
  size_t otherId;
};

struct Station {
  size_t id;
  size_t osmid;
  size_t origGroup, group;
  util::geo::DPoint pos;
  OsmAttrs attrs;
  std::vector<AttrErr> attrErrs;
};

struct Group {
  size_t id;
  size_t osmid;
  util::geo::DPolygon poly;
  OsmAttrs attrs;
  std::vector<size_t> stations;
};

class StatIdx {
 public:
  StatIdx() {}

  void readFromFile(const std::string& path);

  std::vector<const Station*> getStations(const util::geo::DBox bbox) const;
  std::vector<const Group*> getGroups(const util::geo::DBox bbox) const;

  const Station* getStation(size_t id) const;
  const Group* getGroup(size_t id) const;

 private:
  void addStation(size_t id, double lat, double lng, size_t origGroup,
                  size_t group, const OsmAttrs& attrs);
  void addGroup(size_t id, const OsmAttrs& attrs);

  void initIndex();
  void initGroups();

  std::vector<Station> _stations;
  std::vector<Group> _groups;
  util::geo::DBox _bbox;

  util::geo::Grid<size_t, util::geo::Point, double> _sgrid;
  util::geo::Grid<size_t, util::geo::Polygon, double> _ggrid;
};
}  // namespace osmfixer

#endif  // OSMFIXER_INDEX_STATIDX_H_
